# -*- coding: utf-8 -*-

import sys
import os.path
import pwd
import grp
import pathlib


def get_path_owner(apath):
    apath = str(apath)
    if os.path.islink(apath):
        return pwd.getpwuid(os.lstat(apath).st_uid).pw_name
    else:
        return pwd.getpwuid(os.stat(apath).st_uid).pw_name


def get_path_group(apath):
    apath = str(apath)
    if os.path.islink(apath):
        return grp.getgrgid(os.lstat(apath).st_gid).gr_name
    else:
        return grp.getgrgid(os.stat(apath).st_gid).gr_name


def chown(apath, uid, gid):
    apath = str(apath)
    if os.path.islink(apath):
        os.lchown(apath, uid, gid)
    else:
        os.chown(apath, uid, gid)


def normal_path(apath):
    apath = os.path.abspath(str(apath))
    if sys.platform == 'win32':
        apath = apath.replace("/", "\\")
    else:
        apath = apath.replace("\\", "/")
    return pathlib.Path(apath)


def compute_related_path(link, target):
    link = pathlib.Path(normal_path(str(link)))
    target = pathlib.Path(normal_path(str(target)))

    link_parents = [str(p) for p in reversed(link.parents)]
    target_parents = [str(p) for p in reversed(target.parents)]

    max_parents = min(len(link.parents), len(target.parents))
    found_index = max_parents
    for i in range(0, max_parents):
        if link_parents[i] != target_parents[i]:
            found_index = i
            break

    if len(link.parents) > len(target.parents):
        return os.path.join(
            "../" * (len(link.parents) - found_index), target.name)
    elif found_index == max_parents:
        return target.name
    else:
        # Don't join like this:
        #
        #    os.path.join(*target_parents[found_index:], target.name)
        #
        # This syntax will lead python below v3.4 report error : "SyntaxError:
        # only named arguments may follow *expression"!
        return os.path.join(
            os.path.join(*target_parents[found_index:]), target.name)



# -*- coding: utf-8 -*-

import os
import os.path
import sys
import click
import six
import io
import tempfile
import pathlib
import glob
import shutil
import fnmatch
import yaml
import zipfile
import gzip
import stat
import pwd
import grp
import copy
from . import arecabackup
from .pathutils import get_path_owner, get_path_group, chown, normal_path, \
    compute_related_path
from whichcraft import which


class UserData(object):

    def __init__(self):
        pass


# WARNING! When using zip compress (file_compression=True), we got this error
# when checking:
#
# java.lang.IllegalArgumentException: Cannot open an OutputStream in 'append'
# mode on a compressed FileSystem.

def get_project_template():
    from jinja2 import Environment, PackageLoader

    loader = PackageLoader('abhealer', 'templates')

    env = Environment(loader=loader)

    return env.get_template("project.bcfg")


def folder_to_int(name):
    parts = name.split("_")
    first_part = int(parts[0])
    if len(parts) < 2:
        second_part = 0
    else:
        second_part = int(parts[1])

    return first_part * 1000 + second_part


def int_to_folder(aint):
    first_part = aint // 1000
    second_part = aint % 1000

    second_part_text = ""
    if second_part > 0:
        second_part_text = "_%s" % second_part

    return str(first_part) + second_part_text


def find_data_dirs(proj_dir):
    data_dirs = []
    proj_dir = os.path.realpath(str(proj_dir))
    for afolder in os.listdir(proj_dir):
        if afolder.endswith("_data"):
            continue

        if afolder.lower() == "history":
            continue

        data_dirs.append(afolder)

    data_dirs = sorted(data_dirs, key=lambda x: folder_to_int(x))
    data_dirs = [(adir + "_data") for adir in data_dirs]

    return data_dirs


def get_trace_infos(proj_dir):
    data_dirs = find_data_dirs(proj_dir)

    trace_infos = dict()
    for adir in data_dirs:
        info_zip_path = os.path.join(str(proj_dir), adir, "trace")
        with zipfile.ZipFile(info_zip_path) as info_zip_file:
            trace_file = io.BytesIO(info_zip_file.read("trace"))

        with gzip.GzipFile(fileobj=trace_file) as trace_zip_file:
            trace_info = trace_zip_file.read().decode()

        # Parse trace info
        lines = trace_info.splitlines()
        for aline in lines:
            aline = aline.strip()
            if not aline:
                continue

            if aline.startswith('#'):
                continue

            infos = aline.split(';')
            trace_infos[infos[0]] = infos

    return trace_infos


def recover_dirs(is_dockerized, orig_dir, source_dir, dest_dir):

    source_dir = os.path.realpath(str(source_dir))
    dest_dir = os.path.realpath(str(dest_dir))

    if is_dockerized:
        client_source_dir = "/opt/source"
    else:
        client_source_dir = orig_dir

    trace_infos = get_trace_infos(dest_dir)

    for k, v in trace_infos.items():
        # Only process directories
        if not k.startswith("d") and not k.startswith("s"):
            continue

        source_path = pathlib.Path(source_dir) / k[1:]
        client_source_path = pathlib.Path(client_source_dir) / k[1:]

        if k.startswith("d"):
            mode_index = 2
            owner_index = 3
            group_index = 4
        elif k.startswith("s"):
            mode_index = 3
            owner_index = 4
            group_index = 5

        required_mode = int(v[mode_index]) & 0o777

        if k.startswith("d"):

            if source_path.exists():
                if (source_path.stat()[stat.ST_MODE] & 0o777) != required_mode:
                    source_path.chmod(required_mode)
            else:
                source_path.mkdir(mode=required_mode)

        elif k.startswith("s"):
            client_link_path = v[1][1:]

            if client_link_path.startswith(client_source_dir):
                source_path.unlink()
                target = compute_related_path(
                    client_source_path, client_link_path)
                source_path.symlink_to(target)
            else:
                source_path.unlink()
                source_path.symlink_to(client_link_path)

        if ((get_path_owner(source_path) != v[owner_index])
                or (get_path_group(source_path) != v[group_index])):
            chown(
                str(source_path),
                pwd.getpwnam(v[owner_index]).pw_uid,
                grp.getgrnam(v[group_index]).gr_gid)

    print("Recover directories permissions completed!")


def clear_dirs(dest_dir):
    # Clear empty backups that only have
    dirs = glob.glob(os.path.join(str(dest_dir), "*"))
    for adir in dirs:
        if not os.path.isdir(adir):
            continue

        adir = pathlib.Path(adir)
        data_dir = adir.parent / (adir.stem + "_data" + adir.suffix)

        if fnmatch.fnmatch(adir.stem, "*_data"):
            continue

        if not data_dir.exists():
            print("Removed damaged empty backup directory : %s" % adir)
            shutil.rmtree(str(adir), ignore_errors=True)
            continue

        # files = os.listdir(str(adir))

        # There only have .areca-empty file !

        # We must not remove the version that only have .areca-empty changes,
        # otherwise we can't restore the files.

        # Until we find out what's make this not working, MUST not enable
        # codes below:

        # if len(files) <= 1:
        #     # Removed *_data directory first, so we know if a directory
        #     # successed be removed last time.
        #     print("Removing empty backup directory : %s" % adir)
        #     shutil.rmtree(str(data_dir), ignore_errors=True)
        #     shutil.rmtree(str(adir), ignore_errors=True)


def exec_(is_backup, is_dockerized, vars):
    vars = copy.deepcopy(vars)

    temp_dir = tempfile.TemporaryDirectory(prefix="abhealer")

    template = get_project_template()

    source_dir = str(vars["src_path"])

    # Get the parent dir and join with config file base name, don't use
    # specificed dest name
    dest_dir = pathlib.Path(vars["repository"]) / vars["project_name"]

    # If dest dir not existed, we must create it.
    if not dest_dir.exists():
        dest_dir.mkdir(parents=True)

    if not is_backup:
        if not os.path.exists(source_dir):
            os.makedirs(source_dir, exist_ok=True)

        if os.listdir(source_dir):
            raise click.UsageError(
                "Destination must be empty directory!")

    if is_dockerized:
        source_client_dir = "/opt/source"
        dest_client_dir = "/opt/backup"
        workspace_client_dir = "/opt/workspace"
    else:
        source_client_dir = os.path.abspath(source_dir)
        dest_client_dir = os.path.abspath(str(dest_dir))
        workspace_client_dir = os.path.abspath(temp_dir.name)

    fixed_config_file_name = vars["project_name"] + ".bcfg"
    fixed_config_file_path = os.path.join(
        temp_dir.name, fixed_config_file_name)
    fixed_config_file_client_path = os.path.join(
        workspace_client_dir, fixed_config_file_name)

    # Change paths
    vars["src_path"] = source_client_dir
    vars["dst_path"] = dest_client_dir

    docker_image = "starofrainnight/areca-backup"

    # To fix areca won't detect delete changes only
    areca_empty_file_path = os.path.join(source_dir, ".areca-empty")
    pathlib.Path(areca_empty_file_path).touch()

    backup_script_file_name = "_backup.sh"
    backup_script_file_path = os.path.join(
        temp_dir.name, backup_script_file_name)

    config_file_client_path = os.path.join(
        workspace_client_dir, fixed_config_file_name)
    backup_script_file_client_path = os.path.join(
        workspace_client_dir, backup_script_file_name)

    with temp_dir:
        if is_dockerized:
            areca_cl_script_dir = "/usr/local/bin"
        else:
            areca_cl_script_dir = which("areca_cl.sh")
            areca_cl_script_dir = os.path.dirname(areca_cl_script_dir)

        with open(backup_script_file_path, "w") as f:
            f.write("#!/bin/sh\n")
            f.write("cd %s\n" % areca_cl_script_dir)
            if is_backup:
                f.write("./areca_cl.sh backup -config %s -wdir %s\n" % (
                    config_file_client_path, workspace_client_dir))
            else:
                f.write("./areca_cl.sh recover "
                        "-config %s -destination %s -o -nosubdir\n" % (
                            config_file_client_path, source_client_dir))
            f.write("\n")
        os.system("chmod +x %s" % backup_script_file_path)

        print("===== script begin =====")
        with open(backup_script_file_path, "rb") as f:
            print(f.read().decode("utf-8"))
        print("===== script end =====")

        # Write fixed config file
        xml_content = template.render(vars)
        with open(fixed_config_file_path, "wb") as f:
            f.write(xml_content.encode("utf-8"))

        print("===== xml begin =====")
        print(xml_content)
        print("===== xml end =====")

        run_client_cmd = backup_script_file_client_path

        if is_dockerized:
            volume_options = ""
            # Map users and groups
            volume_options += " -v /etc/passwd:/etc/passwd "
            volume_options += " -v /etc/group:/etc/group "
            volume_options += " -v %s:%s " % (
                os.path.abspath(temp_dir.name), workspace_client_dir)
            volume_options += " -v %s:%s " % (
                os.path.abspath(source_dir), source_client_dir)
            volume_options += " -v %s:%s " % (
                dest_dir.resolve(), dest_client_dir)

            backup_cmd = "docker run -t --rm %s %s %s" % (
                volume_options, docker_image, run_client_cmd)
        else:
            backup_cmd = run_client_cmd

        clear_dirs(dest_dir)

        print("Executing : %s" % backup_cmd)

        os.system(backup_cmd)

        if not is_backup:
            recover_dirs(
                is_dockerized, vars["orig_path"], source_dir, dest_dir)

        # Don't remove empty dirs, they are valid either !
        clear_dirs(dest_dir)

    return 0


@click.group()
@click.option(
    '-m', '--mode',
    type=click.Choice(['auto', 'local', 'docker']),
    help='''How we search areca_cl.sh, if mode=auto we will search in path and \
use docker version if mode=docker.''',
    default='auto')
@click.pass_context
def main(ctx, mode):
    """
    This program is a helper for dockerred Areca Backup.

    """

    """
    Areca Backup won't record the empty directories and their properties
    """

    ctx.obj = UserData()

    if mode == "auto":
        areca_cl_path = which("areca_cl.sh")
        ctx.obj.is_dockerized = (areca_cl_path is None)

        if areca_cl_path is None:
            click.echo(
                "Can't found areca console script, use dockerized areca ...")
        else:
            click.echo("Found areca at : %s" % areca_cl_path)
    else:
        ctx.obj.is_dockerized = (mode == "docker")


@main.command()
@click.argument('config', type=click.File())
@click.pass_context
def backup(ctx, config):
    """
    Backup a series projects to repository.

    Project and repository paths are defined in config file.

    \b
    CONFIG: The config file (in YAML format) path.
    """

    vars = yaml.load(config)
    for source in vars["sources"]:
        # Support path only source
        if isinstance(source, six.string_types):
            source = [source]

        vars["src_path"] = source[0]
        vars["project_name"] = os.path.splitext(
            os.path.basename(source[0]))[0]
        if (len(source) > 1) and (len(source[1].strip()) > 0):
            vars["project_name"] = source[1].strip()

        ret = exec_(True, ctx.obj.is_dockerized, vars)
        if ret:
            return ret

    return 0


@main.group()
@click.pass_context
def recover(ctx):
    """
    Recover projects from backup repository.
    """
    pass


@recover.command()
@click.argument('config', type=click.File())
@click.argument('name')
@click.argument('to_path')
@click.pass_context
def proj(ctx, config, name, to_path):
    """
    Only recover specific project

    \b
    CONFIG  : The config file (in YAML format) path
    NAME    : Project name
    TO_PATH : Where you store the recovered project
    """

    vars = yaml.load(config)

    for source in vars["sources"]:
        # Support path only source
        if isinstance(source, six.string_types):
            source = [source]

        vars["project_name"] = os.path.splitext(
            os.path.basename(source[0]))[0]

        if (len(source) > 1) and (len(source[1].strip()) > 0):
            vars["project_name"] = source[1].strip()

        click.echo("%s to %s" % (vars["project_name"], name))
        if vars["project_name"] != name:
            continue

        vars["src_path"] = to_path

        ret = exec_(False, ctx.obj.is_dockerized, vars)
        if ret:
            return ret

        # Only exactly one project with spectific name.
        return 0

    # If no project founded, we should raise error
    raise click.BadArgumentUsage("Project \"%s\" not found!" % name)


@recover.command()
@click.argument('config', type=click.File())
@click.argument('to_path')
@click.pass_context
def repo(ctx, config, to_path):
    """
    Recover whole repository

    \b
    CONFIG  : The config file (in YAML format) path
    TO_PATH : Where you store the recovered project
    """

    vars = yaml.load(config)
    for source in vars["sources"]:
        # Support path only source
        if isinstance(source, six.string_types):
            source = [source]

        vars["project_name"] = os.path.splitext(
            os.path.basename(source[0]))[0]

        if (len(source) > 1) and (len(source[1].strip()) > 0):
            vars["project_name"] = source[1].strip()

        vars["src_path"] = os.path.join(
            to_path, vars["project_name"])

        vars["orig_path"] = os.path.realpath(os.path.normpath(source[0]))

        if exec_(False, ctx.obj.is_dockerized, vars):
            break

    return 0

if __name__ == "__main__":
    # execute only if run as a script
    main()



# -*- coding: utf-8 -*-

'''
This module contained a series classes that use for maintain informations of
Areca Backup.
'''

import pathlib
import arrow
import zipfile
import gzip
import io
import os
import os.path
import subprocess
import xml.etree.ElementTree as etree
from whichcraft import which
from pathlib import Path
from . import abhealer, pathutils


def folder_to_int(name):
    parts = name.split("_")
    first_part = int(parts[0])
    if len(parts) < 2:
        second_part = 0
    else:
        second_part = int(parts[1])

    return first_part * 1000 + second_part


def int_to_folder(aint):
    first_part = aint // 1000
    second_part = aint % 1000

    second_part_text = ""
    if second_part > 0:
        second_part_text = "_%s" % second_part

    return str(first_part) + second_part_text


class TraceInfo(object):

    def __init__(self, info):
        self._parties = info.split(';')

        atype = self.type_

        if atype == "f":
            mode_index = 4
            owner_index = 5
            group_index = 6
        elif atype == "d":
            mode_index = 2
            owner_index = 3
            group_index = 4
        elif atype == "s":
            mode_index = 3
            owner_index = 4
            group_index = 5

        self._mode = int(self._parties[mode_index])
        self._owner = self._parties[owner_index]
        self._group = self._parties[group_index]

    @property
    def type_(self):
        return self._parties[0][0]

    @property
    def path(self):
        return self._parties[0][1:]

    @property
    def mode(self):
        return self._mode

    @property
    def owner(self):
        return self._owner

    @property
    def group(self):
        return self._group

    def __repr__(self):
        return "%s(\"%s\", \"%s\")" % (
            type(self).__qualname__, self.type_, self.path)


class DataInfo(object):
    DIR_SUFFIX = "_data"

    def __init__(self, adir):
        self._base_dir = adir

    def _extract_data(self, name):
        info_zip_path = self.base_dir / name
        with zipfile.ZipFile(str(info_zip_path)) as info_zip_file:
            data_file = io.BytesIO(info_zip_file.read(name))

        with gzip.GzipFile(fileobj=data_file) as data_zip_file:
            data = data_zip_file.read().decode()

        return data

    @property
    def base_dir(self):
        return self._base_dir

    @property
    def datetime(self):
        value = str(int(self))
        return arrow.Arrow(
            int(value[:4]),
            int(value[4:6]),
            int(value[6:8]),
            int(value[8:10]),
            int(value[10:12]),
            0,
            int(value[12:]),
        ).datetime

    @property
    def traces(self):
        infos = list()

        data = self._extract_data("trace")

        # Parse trace info
        lines = data.splitlines()
        for aline in lines:
            aline = aline.strip()
            if not aline:
                continue

            if aline.startswith('#'):
                continue

            info = TraceInfo(aline)
            infos.append(info)

        return infos

    @property
    def manifest(self):
        """
        Get manifest of this backup data info.

        Return a root element of ElementTree about the xml content.

        Sample manifest content:

        <?xml version="1.0" encoding="UTF-8"?>
        <manifest version="1" type="0" date="2017_11_03 20h56-38-284">
        <properties>
        <property key="Archive name" value="201711032056" />
        <property key="Archive size" value="256" />
        <property key="Areca Home" value="/home/useradmin/applications/areca/bin" />
        <property key="Backup duration" value="273 ms" />
        <property key="Build id" value="5872222636083894532" />
        <property key="Checked" value="false" />
        <property key="File encoding" value="UTF-8" />
        <property key="Filtered entries" value="0" />
        <property key="JRE" value="/usr/lib/jvm/java-8-openjdk-amd64/jre" />
        <property key="Operating system" value="Linux - 4.4.0-34-generic" />
        <property key="Option [backup scheme]" value="Incremental backup" />
        <property key="Scanned entries (files or directories)" value="2" />
        <property key="Source path" value="/mnt/data/sources/projects/abhealer/from" />
        <property key="Stored files" value="1" />
        <property key="Target ID" value="from" />
        <property key="Unfiltered directories" value="1" />
        <property key="Unfiltered files" value="1" />
        <property key="Unmodified files (not stored)" value="0" />
        <property key="Version" value="7.5" />
        <property key="Version date" value="August 26, 2015" />
        </properties>
        </manifest>
        """

        data = self._extract_data("manifest")
        return etree.fromstring(data)

    def _name_without_suffix(self):
        return self.base_dir.name[:-len(self.DIR_SUFFIX)]

    def __repr__(self):
        return "%s(%s)" % (type(self).__qualname__, int(self))

    def __int__(self):
        return folder_to_int(self._name_without_suffix())


class Project(object):

    def __init__(self, repository, cfg, adir):
        self._base_dir = Path(adir)
        self._repository = repository
        self._cfg = cfg

    @property
    def repository(self):
        return self._repository

    @property
    def base_dir(self):
        return self._base_dir

    @property
    def name(self):
        return self.base_dir.name

    @property
    def data_infos(self):
        infos = []
        for subdir in self.base_dir.iterdir():
            if not subdir.name.endswith(DataInfo.DIR_SUFFIX):
                continue

            infos.append(DataInfo(subdir))

        infos = sorted(infos, key=lambda x: int(x))

        return infos

    def __repr__(self):
        return "%s(\"%s\")" % (type(self).__qualname__, self.name)


class Repository(object):
    CFG_DIR_NAME = "areca_config_backup"

    def __init__(self, adir):
        self._base_dir = Path(adir)

        # Validate directory
        self._cfg_dir = self._base_dir / self.CFG_DIR_NAME
        if not self._cfg_dir.exists():
            msg = ("'%s' directory not found ! The directory "
                   "is not an Areca Backup repository: %s!")
            msg = msg % (self.CFG_DIR_NAME, self._base_dir)
            raise NotADirectoryError(msg)

    @property
    def base_dir(self):
        return self._base_dir

    @property
    def cfg_dir(self):
        return self._cfg_dir

    @property
    def projects(self):

        all = []
        for subdir in self.base_dir.iterdir():
            if subdir.name == self.CFG_DIR_NAME:
                continue

            history_path = subdir / "history"
            if (not history_path.exists()) or (not history_path.is_file()):
                abhealer.logger().warn(
                    "Found invalid project : %s" % subdir.name)
                continue

            cfg_path = self.cfg_dir / (subdir.name + ".bcfg")
            all.append(Project(
                self, etree.parse(str(cfg_path)), subdir))

        return all


class LocalArecaBackup(object):
    '''
    The class use for maintain Areca Backup's behaviors.
    '''

    def __init__(self):
        # Search areca backup script in PATH
        self._program_path = self._detect_program_path()

    def _detect_program_path(self):
        apath = which("areca_cl.sh")
        if apath is None:
            return None

        return Path(apath)

    def backup(self, cfg_path, ws_dir):
        return subprocess.call(
            self.gen_backup_cmd(cfg_path, ws_dir), shell=True)

    def recover(self, cfg_path, dst_dir):
        return subprocess.call(
            self.gen_recover_cmd(cfg_path, dst_dir), shell=True)

    def gen_backup_cmd(self, cfg_path, ws_dir=None):
        """
        Generate backup command

        :param ws_dir: Working directory during archive check. Default to None
        for use cfg_path's path
        """
        cfg_path = pathutils.normal_path(cfg_path)

        if ws_dir:
            ws_dir = pathutils.normal_path(ws_dir)
        else:
            ws_dir = pathutils.normal_path(cfg_path).parent

        cmd = "cd %s; ./areca_cl.sh backup -config %s -wdir %s"
        cmd = cmd % (self._program_path.parent,
                     str(cfg_path), str(ws_dir))
        return cmd

    def gen_recover_cmd(self, cfg_path, dst_dir):

        cfg_path = pathutils.normal_path(cfg_path)
        dst_dir = pathutils.normal_path(dst_dir)

        cmd = ("cd %s; ./areca_cl.sh recover -config %s -destination %s "
               "-o -nosubdir")
        cmd = cmd % (self._program_path.parent,
                     str(cfg_path), str(dst_dir))

        return cmd


class DockerizedArecaBackup(LocalArecaBackup):
    '''
    The class use for maintain Dockerized Areca Backup's behaviors.
    '''

    CMD_PREFIX = "docker run -t --rm starofrainnight/areca-backup "
    SRC_DIR = "/opt/source"
    DST_DIR = "/opt/backup"
    WS_DIR = "/opt/workspace"
    CFG_DIR = "/opt/config"

    def __init__(self):
        super().__init__()

    def _detect_program_path(self):
        cmd = "%s bash --version" % self.CMD_PREFIX
        ret = subprocess.call(cmd, shell=True)
        if ret:
            return None

        return Path("/usr/local/bin/areca_cl.sh")

    def gen_docker_volume_options(self, cfg_path, dst_dir):
        pass

    def gen_backup_cmd(self, cfg_path, ws_dir=None):
        cfg_path = pathutils.normal_path(cfg_path)
        client_cfg_path = os.path.join(self.CFG_DIR, cfg_path.name)

        if ws_dir:
            ws_dir = pathutils.normal_path(ws_dir)
        else:
            ws_dir = pathutils.normal_path(cfg_path).parent

        cmd = super().gen_backup_cmd(client_cfg_path, self.WS_DIR)

        return cmd

    def gen_recover_cmd(self, cfg_path, dst_dir):
        cfg_path = pathutils.normal_path(cfg_path)
        client_cfg_path = os.path.join(self.CFG_DIR, cfg_path.name)

        dst_dir = pathutils.normal_path(dst_dir)

        cmd = super().gen_recover_cmd(client_cfg_path, self.DST_DIR)

        return cmd



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.rst') as readme_file, open('HISTORY.rst') as history_file:
    long_description = (readme_file.read() + "\n\n" + history_file.read())

requirements = [
    'click>=6.0',
    'jinja2',
    'pyyaml',
    'arrow',
    'whichcraft',
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='abhealer',
    version='0.5.0',
    description="abhealer",
    long_description=long_description,
    author="Hong-She Liang",
    author_email='starofrainnight@gmail.com',
    url='https://github.com/starofrainnight/abhealer',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'abhealer=abhealer.__main__:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="Apache Software License",
    zip_safe=False,
    keywords='abhealer',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
